ReprPtr

Struct ReprPtr 

pub struct ReprPtr<R, T> {
    pub ptr: PPtr<R>,
    pub _T: PhantomData<T>,
}
Concrete representation of a pointer to an object of type T with representation type R The length of the array is not stored in the pointer

Fields

ptr: PPtr<R>_T: PhantomData<T>

Implementations

Source

impl<R, T> ReprPtr<R, T>

pub open spec fn new_spec(ptr: PPtr<R>) -> Self

{ Self { ptr, _T: PhantomData } }
pub exec fn from_pptr(ptr: PPtr<R>) -> res : Self

ensures
res == Self::new_spec(ptr),
res.addr() == ptr.addr(),
res.ptr == ptr,
pub open spec fn to_pptr(self) -> PPtr<R>

{ self.ptr }
pub open spec fn addr_spec(self) -> usize

{ self.ptr.addr() }
pub exec fn addr(self) -> usize

returns
self.addr(),
Source

impl<R, T: Repr<R>> ReprPtr<R, T>

pub exec fn take(self, Tracked(perm): Tracked<&mut PointsTo<R, T>>) -> v : T

requires
old(perm).pptr() == self,
old(perm).is_init(),
old(perm).wf(&old(perm).inner_perms),
ensures
final(perm).pptr() == old(perm).pptr(),
final(perm).mem_contents() == MemContents::Uninit::<T>,
v == old(perm).value(),
final(perm).inner_perms == old(perm).inner_perms,
pub exec fn put(self, Tracked(perm): Tracked<&mut PointsTo<R, T>>, v: T)

requires
old(perm).pptr() == self,
old(perm).mem_contents() == MemContents::Uninit::<T>,
ensures
final(perm).pptr() == old(perm).pptr(),
final(perm).mem_contents() == MemContents::Init(v),
final(perm).wf(&final(perm).inner_perms),
final(perm).inner_perms == v.to_repr_spec(old(perm).inner_perms).1,
final(perm).points_to.value() == v.to_repr_spec(old(perm).inner_perms).0,
final(perm).points_to.is_init(),
pub exec fn borrow<'a>(self, Tracked(perm): Tracked<&'a PointsTo<R, T>>) -> v : &'a T

requires
perm.pptr() == self,
perm.is_init(),
perm.wf(&perm.inner_perms),
ensures
*v == perm.value(),
pub exec fn borrow_mut<'a>( self, Tracked(perm): Tracked<&'a mut PointsTo<R, T>>, ) -> v : &'a mut T

requires
old(perm).pptr() == self,
old(perm).is_init(),
old(perm).wf(&old(perm).inner_perms),
ensures
*v == old(perm).value(),
final(perm).pptr() == old(perm).pptr(),
final(perm).is_init(),
final(perm).wf(&final(perm).inner_perms),
final(perm).value() == *final(v),

Borrows the pointed-to T mutably for the lifetime of perm.

While the returned borrow is live, perm is exclusively held and cannot be used. The returned borrow is tied to the tracked permission: at the return point, the permission’s value matches *v. Callers must preserve any invariants beyond the final initialised/well-formed state themselves.

FIXME[SOUNDNESS]: This method is unsound because the caller must ensure the final value of v is consistent with the inner permissions.